package com.zipcodewilmington.gildedrose;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class InventoryTest {

    @Test
    public void setNameTest(){
        //Given
        Item candle = new Item("Scented Candle", 30, 40);
        String expected = "Candle";

        //When
        candle.setName("Candle");

        //Then
        String actual = candle.getName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void setQualityTest(){
        //Given
        Item candle = new Item("Scented Candle", 30, 40);
        int expected = 25;

        //When
        candle.setQuality(25);

        //Then
        int actual = candle.getQuality();
        Assert.assertEquals(expected, actual);
    }


    @Test
    public void getNameTest(){
        //Given
        String expected = "Scented Candle";

        //When
        Item candle = new Item("Scented Candle", 30, 40);

        //Then
        String actual = candle.getName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void getSellInTest(){
        //Given
        int expected = 30;

        //When
        Item candle = new Item("Scented Candle", 30, 40);

        //Then
        int actual = candle.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void getQualityTest(){
        //Given
        int expected = 40;

        //When
        Item candle = new Item("Scented Candle", 30, 40);

        //Then
        int actual = candle.getQuality();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sulfurasUpdateSellInTest(){
        //Given
        Item sulfuras = new Item("Sulfuras", 0, 80);
        Item[] itemArr = new Item[1];
        itemArr[0] = sulfuras;
        Inventory newInv = new Inventory(itemArr);
        int expected = 0;

        //When
        newInv.updateSellIn();

        //Then
        int actual = sulfuras.getSellIn();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void itemAtZeroUpdateSellInTest(){
        //Given
        Item wine = new Item("Wine", 0, 40);
        Item[] itemArr = new Item[1];
        itemArr[0] = wine;
        Inventory newInv = new Inventory(itemArr);
        int expected = 0;

        //When
        newInv.updateSellIn();

        //Then
        int actual = wine.getSellIn();

        Assert.assertEquals(expected, actual);
    }



    @Test
    public void allOthersUpdateSellInTest(){
        //Given
        Item candle = new Item("Scented Candle", 30, 40);
        Item[] itemArr = new Item[1];
        itemArr[0] = candle;
        Inventory newInv = new Inventory(itemArr);
        int expected = 29;

        //When
        newInv.updateSellIn();

        //
        int actual = candle.getSellIn();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sulfurasUpdateQualityTest(){
        //Given
        Item sulfuras = new Item("Sulfuras", 0, 80);
        Item[] itemArr = new Item[1];
        itemArr[0] = sulfuras;
        Inventory newInv = new Inventory(itemArr);
        int expected = 80;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = sulfuras.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void backstagePasses9DaysUpdateQualityTest(){
        //Given
        Item backstagePass = new Item("Backstage Passes", 9, 10);
        Item[] itemArr = new Item[1];
        itemArr[0] = backstagePass;
        Inventory newInv = new Inventory(itemArr);
        int expected = 12;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = backstagePass.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void backstagePasses3DaysUpdateQualityTest(){
        //Given
        Item backstagePass = new Item("Backstage Passes", 3, 10);
        Item[] itemArr = new Item[1];
        itemArr[0] = backstagePass;
        Inventory newInv = new Inventory(itemArr);
        int expected = 13;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = backstagePass.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void backstagePasses0DaysUpdateQualityTest(){
        //Given
        Item backstagePass = new Item("Backstage Passes", 1, 10);
        Item[] itemArr = new Item[1];
        itemArr[0] = backstagePass;
        Inventory newInv = new Inventory(itemArr);
        int expected = 0;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = backstagePass.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void agedBrieUpdateQualityTest(){
        //Given
        Item agedBrie = new Item("Aged Brie", 20, 5);
        Item[] itemArr = new Item[1];
        itemArr[0] = agedBrie;
        Inventory newInv = new Inventory(itemArr);
        int expected = 6;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = agedBrie.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sellinIs0UpdateQualityTest(){
        //Given
        Item candle = new Item("Scented Candle", 0, 40);
        Item[] itemArr = new Item[1];
        itemArr[0] = candle;
        Inventory newInv = new Inventory(itemArr);
        int expected = 38;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = candle.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void defaultUpdateQualityTest(){
        //Given
        Item candle = new Item("Scented Candle", 30, 40);
        Item[] itemArr = new Item[1];
        itemArr[0] = candle;
        Inventory newInv = new Inventory(itemArr);
        int expected = 39;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = candle.getQuality();

        Assert.assertEquals(expected, actual);
    }

    @Test
    public void ConjuredQualityTest(){
        //Given
        Item conjured = new Item("Conjured", 10, 20);
        Item[] itemArr = new Item[1];
        itemArr[0] = conjured;
        Inventory newInv = new Inventory(itemArr);
        int expected = 18;

        //When
        newInv.updateSellIn();
        newInv.updateQuality();

        //Then
        int actual = conjured.getQuality();

        Assert.assertEquals(expected, actual);
    }
}
